@@ -27,6 +27,7 @@ Functions
    DBBact.plot_term_annotations_venn
    DBBact.plot_term_venn_all
    DBBact.sample_enrichment
+   DBBact.get_wordcloud_stats
    DBBact.draw_wordcloud
    DBBact.get_enrichment_score
    DBBact.show_enrichment_qt5
@@ -94,6 +95,16 @@ class DBBact(Database):
         '''
         return __version__
 
+    def set_log_level(self, level='WARNING'):
+        '''Set the log level for the dbbact-calour messages
+
+        Parameters
+        ----------
+        level: str or int, optional
+            can be 'DEBUG'/'INFO'/'WARNING'/'ERROR' or int in the range of 0 (full debug) to 50 (only errors)
+        '''
+        logger.setLevel(level)
+
     def get_seq_annotation_strings(self, *kargs, **kwargs):
         '''Get a list of strings describing the sequence annotations, and the annotations details
 
@@ -251,12 +262,12 @@ class DBBact(Database):
             not None to store results in the exp (to save time for multiple queries)
         term_type : str or None (optional)
             The type of terms to return. optional values:
-            None to use default ('seqstring')
+            None to use default ('fscore')
             'annotation': the annotation string for each annotation (i.e. 'higher in fish compared to dogs...')
             'terms': the ontology terms without parent terms (with '-' attached to negative freq. terms)
             'parentterms': the ontology terms including all parent terms (with '-' attached to negative freq. terms)
             'contamination': get if bacteria is contaminant or not
-            'fscore': ...
+            'fscore': get the fscore (combination of recall and precision) for each term
         ignore_exp : list of int or None (optional)
             the list of experimentids to ignore (don't take info from them)
         term_method: list of str, optional
@@ -275,7 +286,7 @@ class DBBact(Database):
             key is the feature, value is a dict of the score (value) for each term(key) for this feature
         '''
         if term_type is None:
-            term_type = 'terms'
+            term_type = 'fscore'
         if exp is not None:
             # if annotations not yet in experiment - add them
             self.add_all_annotations_to_exp(exp, max_id=max_id, force=False, **kwargs)
@@ -334,9 +345,8 @@ class DBBact(Database):
                     #     newdesc.append(cterm)
                     #     term_scores[cseq][cterm] += 1
                 new_annotations[cseq] = annotation_terms
-
         # f-score for each term
-        elif term_type == 'fscore':
+        elif term_type == 'fscore' or term_type == 'recall' or term_type == 'precision':
             if ignore_exp is None:
                 ignore_exp = []
             # we need to rekey the annotations with an str (old problem...)
@@ -346,12 +356,18 @@ class DBBact(Database):
                     continue
                 term_scores[cseq] = defaultdict(float)
                 fscore, recall, precision, term_count, reduced_f = get_enrichment_score(annotations, [(cseq, annotations_list)], ignore_exp=ignore_exp, term_info=term_info, term_types=term_method)
-                if len(fscore) == 0:
+                if term_type == 'fscore':
+                    use_score = fscore
+                elif term_type == 'recall':
+                    use_score = recall
+                elif term_type == 'precision':
+                    use_score = precision
+                if len(use_score) == 0:
                     new_annotations[cseq] = ['NA']
                     continue
-                sorted_fscore = sorted(fscore.items(), key=lambda x: x[1], reverse=True)
-                new_annotations[cseq] = [sorted_fscore[0][0]]
-                term_scores[cseq] = fscore
+                sorted_score = sorted(use_score.items(), key=lambda x: x[1], reverse=True)
+                new_annotations[cseq] = [sorted_score[0][0]]
+                term_scores[cseq] = use_score
         elif term_type == 'parentterms':
             for cseq, term_list in sequence_terms.items():
                 if cseq not in features:
@@ -607,6 +623,7 @@ class DBBact(Database):
         -------
         count
         '''
+        raise ValueError('function depracated')
         exp_features = set(exp.feature_metadata.index.values)
         bad_features = set(features).difference(exp_features)
         if len(bad_features) > 0:
@@ -1181,22 +1198,83 @@ class DBBact(Database):
 
         Returns
         -------
-        pandas.DataFrame with  info about significantly enriched terms. columns:
-            feature : str the feature
-            pval : the p-value for the enrichment (float)
-            odif : the effect size (float)
-            observed : the number of observations of this term in group1 (int)
-            expected : the expected (based on all features) number of observations of this term in group1 (float)
-            frac_group1 : fraction of total terms in group 1 which are the specific term (float)
-            frac_group2 : fraction of total terms in group 2 which are the specific term (float)
-            num_group1 : number of total terms in group 1 which are the specific term (float)
-            num_group2 : number of total terms in group 2 which are the specific term (float)
-            description : the term (str)
-        numpy.Array where rows are features (ordered like the dataframe), columns are features and value is score
-            for term in feature
-        pandas.DataFrame with info about the features used. columns:
-            group: int the group (1/2) to which the feature belongs
-            sequence: str
+        calour.Experiment
+            with significanly enriched terms as features (and samples reamin samples).
+            Data indicates the score for term in sample
+            feature_metadata contains 'num_features' field which indicates the number of features that were associated with the term
+
+        # pandas.DataFrame with  info about significantly enriched terms. columns:
+        #     feature : str the feature
+        #     pval : the p-value for the enrichment (float)
+        #     odif : the effect size (float)
+        #     observed : the number of observations of this term in group1 (int)
+        #     expected : the expected (based on all features) number of observations of this term in group1 (float)
+        #     frac_group1 : fraction of total terms in group 1 which are the specific term (float)
+        #     frac_group2 : fraction of total terms in group 2 which are the specific term (float)
+        #     num_group1 : number of total terms in group 1 which are the specific term (float)
+        #     num_group2 : number of total terms in group 2 which are the specific term (float)
+        #     description : the term (str)
+        # numpy.Array where rows are features (ordered like the dataframe), columns are features and value is score
+        #     for term in feature
+        # pandas.DataFrame with info about the features used. columns:
+        #     group: int the group (1/2) to which the feature belongs
+        #     sequence: str
+        '''
+        newexp = self.sample_term_scores(exp, term_type=term_type, ignore_exp=ignore_exp, min_appearances=min_appearances, score_method=score_method, freq_weight=freq_weight, use_term_pairs=use_term_pairs, max_id=max_id, axis='s')
+        # get the differentially abundant terms between the two sample groups
+        dd = newexp.diff_abundance(field, value1, value2, fdr_method=fdr_method, transform='log2data', alpha=alpha)
+        return dd
+
+    def sample_term_scores(self, exp, term_type='term', ignore_exp=None, min_appearances=3, score_method='all_mean', freq_weight='log', use_term_pairs=False, max_id=None, axis='s'):
+        '''Get the list of enriched terms for all bacteria between two groups using frequencies from the Experiment.data table.
+
+        It is equivalent to multiplying the (freq_weight transformed) feature X sample matrix by the database derived term X feature matrix
+        (where the numbers are how strong is the term associated with the feature based on database annotations using score_method).
+        A differntial abundance test (using dsFDR multiple hypothesis correction) is then performed on the resulting sample X term matrix.
+
+        Parameters
+        ----------
+        exp : calour.Experiment
+            The experiment to compare the features to
+        term_type : str or None (optional)
+            The type of annotation data to test for enrichment
+            options are:
+            'term' - ontology terms associated with each feature.
+            'parentterm' - ontology terms including parent terms associated with each feature.
+            'annotation' - the full annotation strings associated with each feature
+            'combined' - combine 'term' and 'annotation'
+            'fscore' - the fscore for each term (recall/precision)
+        ignore_exp: list of int or None or True, optional
+            List of experiments to ignore in the analysis
+            True to ignore annotations from the current experiment
+            None (default) to use annotations from all experiments including the current one
+        min_appearances : int (optional)
+            The minimal number of times a term appears in order to include in output list.
+        score_method : str (optional)
+            The score method used for calculating the term score. Options are:
+            'all_mean' (default): mean over each experiment of all annotations containing the term
+            'sum' : sum of all annotations (experiment not taken into account)
+            'card_mean': use a null model keeping the number of annotations per each bacteria
+        freq_weight : str (optional)
+            How to incorporate the frequency of each feature (in a sample) into the term count for the sample. options:
+                'linear' : use the frequency
+                'log' : use the log2 of the frequency
+                'binary' : use the presence/absence of the feature
+                'rank' : use the rank of each feature across samples
+        Use_term_pairs: bool, optional
+            True to get all term pair annotations (i.e. human+feces etc.)
+        max_id: int or None, optional
+            if not None, limit results to annotation ids <= max_id
+        axis: str or int, optional
+            's' or 1 to return per-sample term scores experiment
+            'f' or 0 to return per-feature term scores experiment
+
+        Returns
+        -------
+        calour.Experiment
+            with terms as features, (is axis='s', samples remain samples, if axis='f' samples are the features).
+            Data indicates the score for term in sample / feature
+            feature_metadata contains 'num_features' field which indicates the number of features that were associated with the term
         '''
         exp_features = list(exp.feature_metadata.index.values)
 
@@ -1211,12 +1289,15 @@ class DBBact(Database):
             else:
                 logger.info('Found %d experiments (%s) matching current experiment - ignoring them.' % (len(ignore_exp), ignore_exp))
 
+        # get the term/annotation scores
+        # we store it in feature_terms:
+        # dict of {feature:str, list of tuples of (term:str, score:float)}, key is feature, value is list of (term, score)
         all_annotations = exp.databases['dbbact']['annotations']
         seq_annotations = exp.databases['dbbact']['sequence_annotations']
         if term_type == 'term':
             feature_terms = self.db._get_all_term_counts(exp_features, seq_annotations, all_annotations, ignore_exp=ignore_exp, score_method=score_method, use_term_pairs=use_term_pairs)
         elif term_type == 'parentterm':
-            pass
+            raise ValueError('term_type parentterm not supported yet')
         elif term_type == 'annotation':
             feature_terms = self.db._get_all_annotation_string_counts(exp_features, all_anntations=all_annotations, seq_annotations=seq_annotations, ignore_exp=ignore_exp)
         elif term_type == 'combined':
@@ -1226,10 +1307,22 @@ class DBBact(Database):
                 if cfeature not in feature_terms:
                     feature_terms[cfeature] = []
                 feature_terms[cfeature].extend(cvals)
+        elif term_type == 'fscore':
+            res = self.get_exp_feature_stats(exp, ignore_exp=None, term_info=None, term_types='single', threshold=None, max_id=None)
+            feature_terms = defaultdict(list)
+            for cseq, cres in res.items():
+                scores = cres['fscore']
+                reslist = []
+                for cterm, cscore in scores.items():
+                    reslist.append([cterm, cscore])
+                feature_terms[cseq] = reslist
         else:
             raise ValueError('term_type %s not supported for dbbact. possible values are: "term", "parentterm", "annotation", "combined"')
 
         # get all terms. store the index position for each term
+        # store in:
+        # terms: dict of {term(str): pos(int)} - the intended position (in the new results) of each term
+        # term_features dict of {term(str): count(int)} - the number of features containing each term
         terms = {}
         term_features = defaultdict(int)
         cpos = 0
@@ -1242,11 +1335,8 @@ class DBBact(Database):
 
         logger.debug('found %d terms for all features' % len(terms))
 
-        # create the term x sample array
-        fs_array = np.zeros([exp.data.shape[0], len(terms)])
-
-        data = exp.get_data(sparse=False, copy=True)
         # how to weight the frequency of each bacteria
+        data = exp.get_data(sparse=False, copy=True)
         if freq_weight == 'log':
             data[data < 1] = 1
             data = np.log2(data)
@@ -1258,29 +1348,47 @@ class DBBact(Database):
             logger.debug('ranking the data')
             for ccol in range(np.shape(data)[1]):
                 data[:, ccol] = scipy.stats.rankdata(data[:, ccol])
-
         else:
             raise ValueError('unknown freq_weight option %s. Can use ["log", "binary","linear"].')
 
-        # iterate over all features and add to all terms associated with the feature
-        for idx, cfeature in enumerate(exp_features):
-            fterms = feature_terms[cfeature]
-            for cterm, cval in fterms:
-                fs_array[:, terms[cterm]] += cval * data[:, idx]
+        if axis == 's':
+            # create the sample x term results array
+            fs_array = np.zeros([exp.data.shape[0], len(terms)])
+
+            # iterate over all features and add to all terms associated with the feature
+            for idx, cfeature in enumerate(exp_features):
+                fterms = feature_terms[cfeature]
+                for cterm, cval in fterms:
+                    fs_array[:, terms[cterm]] += cval * data[:, idx]
+
+            # create the new experiment with samples x terms
+            sm = deepcopy(exp.sample_metadata)
+            sorted_term_list = sorted(terms, key=terms.get)
+            fm = pd.DataFrame(data={'term': sorted_term_list}, index=sorted_term_list)
+            fm['num_features'] = [term_features[d] for d in fm.index]
+        elif axis == 'f':
+            # create the sample x term results array
+            fs_array = np.zeros([exp.data.shape[1], len(terms)])
+
+            # iterate over all features and add to all terms associated with the feature
+            for idx, cfeature in enumerate(exp_features):
+                fterms = feature_terms[cfeature]
+                for cterm, cval in fterms:
+                    fs_array[idx, terms[cterm]] = cval
+
+            # create the new experiment with samples x terms
+            sm = deepcopy(exp.feature_metadata)
+            sorted_term_list = sorted(terms, key=terms.get)
+            fm = pd.DataFrame(data={'term': sorted_term_list}, index=sorted_term_list)
+            fm['num_features'] = [term_features[d] for d in fm.index]
+        else:
+            raise ValueError('axis %s not supported' % axis)
 
-        # create the new experiment with samples x terms
-        sm = deepcopy(exp.sample_metadata)
-        sorted_term_list = sorted(terms, key=terms.get)
-        fm = pd.DataFrame(data={'term': sorted_term_list}, index=sorted_term_list)
-        fm['num_features'] = [term_features[d] for d in fm.index]
         newexp = Experiment(fs_array, sample_metadata=sm, feature_metadata=fm, description='Term scores')
-
-        # get the differentially abundant terms between the two sample groups
-        dd = newexp.diff_abundance(field, value1, value2, fdr_method=fdr_method, transform='log2data', alpha=alpha)
-        return dd
+        return newexp
 
     def rank_enrichment(self, exp, field, term_type='term', ignore_exp=None, min_appearances=3, fdr_method='dsfdr', score_method='all_mean', method='spearman', alpha=0.1, use_term_pairs=False, max_id=None):
-        '''Get the list of terms significanly correlated with the values for all bacteria.
+        '''Get the list of terms significanly correlated/anti-correlated with the values in field for all bacteria.
 
         Identify terms correlated with the values by calculating for each term the term score for each feature, and correlating this score/feature vector with the values vector
 
@@ -1329,75 +1437,13 @@ class DBBact(Database):
             with singnificantly correlated terms
             terms are features, sequences are samples
         '''
-        exp_features = list(exp.feature_metadata.index.values)
-
-        # add all annotations to experiment if not already added
-        self.add_all_annotations_to_exp(exp, max_id=max_id, force=False)
-
-        # if ignore exp is True, it means we should ignore the current experiment
-        if ignore_exp is True:
-            ignore_exp = self.db.find_experiment_id(datamd5=exp.info['data_md5'], mapmd5=exp.info['sample_metadata_md5'], getall=True)
-            if ignore_exp is None:
-                logger.warn('No matching experiment found in dbBact. Not ignoring any experiments')
-            else:
-                logger.info('Found %d experiments (%s) matching current experiment - ignoring them.' % (len(ignore_exp), ignore_exp))
-
-        all_annotations = exp.databases['dbbact']['annotations']
-        seq_annotations = exp.databases['dbbact']['sequence_annotations']
-        if term_type == 'term':
-            feature_terms = self.db._get_all_term_counts(exp_features, seq_annotations, all_annotations, ignore_exp=ignore_exp, score_method=score_method, use_term_pairs=use_term_pairs)
-        elif term_type == 'parentterm':
-            pass
-        elif term_type == 'annotation':
-            feature_terms = self.db._get_all_annotation_string_counts(exp_features, all_anntations=all_annotations, seq_annotations=seq_annotations, ignore_exp=ignore_exp)
-        elif term_type == 'combined':
-            feature_terms = self.db._get_all_term_counts(exp_features, seq_annotations, all_annotations, ignore_exp=ignore_exp, score_method=score_method, use_term_pairs=use_term_pairs)
-            feature_annotations = self.db._get_all_annotation_string_counts(exp_features, all_annotations=all_annotations, seq_annotations=seq_annotations, ignore_exp=ignore_exp)
-            for cfeature, cvals in feature_annotations.items():
-                if cfeature not in feature_terms:
-                    feature_terms[cfeature] = []
-                feature_terms[cfeature].extend(cvals)
-        else:
-            raise ValueError('term_type %s not supported for dbbact. possible values are: "term", "parentterm", "annotation", "combined"')
-
-        # get all terms. store the index position for each term
-        terms = {}
-        term_features = defaultdict(int)
-        cpos = 0
-        for cfeature, ctermlist in feature_terms.items():
-            for cterm, ccount in ctermlist:
-                if cterm not in terms:
-                    terms[cterm] = cpos
-                    cpos += 1
-                term_features[cterm] += 1
-
-        logger.debug('found %d terms for all features' % len(terms))
-
-        # create the term x feature array
-        fs_array = np.zeros([exp.data.shape[1], len(terms)])
-
-        # iterate over all features and add to all terms associated with the feature
-        for idx, cfeature in enumerate(exp_features):
-            fterms = feature_terms[cfeature]
-            for cterm, cval in fterms:
-                fs_array[idx, terms[cterm]] = cval
-
-        # create the new experiment with features x terms
-        values = exp.feature_metadata[field].values
-        sm = deepcopy(exp.feature_metadata)
-        sm[field] = values
-        sorted_term_list = sorted(terms, key=terms.get)
-        fm = pd.DataFrame(data={'term': sorted_term_list}, index=sorted_term_list)
-        fm['num_features'] = [term_features[d] for d in fm.index]
-
-        newexp = Experiment(fs_array, sample_metadata=sm, feature_metadata=fm, description='Term scores')
-
+        newexp = self.sample_term_scores(exp, term_type=term_type, ignore_exp=ignore_exp, min_appearances=min_appearances, score_method=score_method, use_term_pairs=use_term_pairs, max_id=max_id, axis='f')
         # get the correlated terms for the value supplied
         dd = newexp.correlation(field, fdr_method=fdr_method, alpha=alpha, method=method)
         return dd
 
-    def draw_wordcloud(self, exp=None, features=None, term_type='fscore', ignore_exp=None, width=2000, height=1000, freq_weighted=False, relative_scaling=0.5, focus_terms=None, threshold=None, max_id=None):
-        '''Draw a word_cloud for a given set of sequences
+    def get_wordcloud_stats(self, exp=None, features=None, ignore_exp=None, freq_weighted=False, focus_terms=None, threshold=None, max_id=None):
+        '''Get recall/precision/f-scores for a set of features
 
         Parameters
         ----------
@@ -1407,24 +1453,14 @@ class DBBact(Database):
         features: list of str or None, optional
             None to use the features from exp. Otherwise, a list of features ('ACGT' sequences) that is a subset of the features in exp (if exp is supplied)
             Note: if exp is None, must provide features.
-        term_type: str, optional
-            What score to use for the word cloud. Options are:
-            'recall': sizes are based on the recall (fraction of dbbact term containing annotations that contain the sequences)
-            'precision': sizes are based on the precision (fraction of sequence annotations of the experiment sequences that contain the term)
-            'fscore': a combination of recall and precition (r*p / (r+p))
         ignore_exp: list of int or None or True, optional
             List of experiments to ignore in the analysis
             True to ignore annotations from the current experiment (if exp is supplied)
             None (default) to use annotations from all experiments including the current one
-        width, height: int, optional
-            The width and heigh of the figure (high values are slower but nicer resolution for publication)
-            If inside a jupyter notebook, use savefig(f, dpi=600)
         freq_weighted: bool, optional
             Only when supplying exp
             True to weight each bacteria by it's mean frequency in exp.data
             NOT IMPLEMENTED YET!!!
-        relative_scaling: float, optional
-            the effect of the score on the word size. 0.5 is a good compromise (passed to wordcloud.Wordcloud())
         focus_terms: list of str or None, optional
             if not None, use only annotations containing all terms in focus_terms list.
             for example, if focus_terms=['homo sapiens', 'feces'], will only draw the wordcloud for annotations of human feces
@@ -1436,15 +1472,8 @@ class DBBact(Database):
 
         Returns
         -------
-        matplotlib.figure
+        fscores, recall, precision, term_count, reduced_f
         '''
-        import matplotlib.pyplot as plt
-        try:
-            from wordcloud import WordCloud
-        except Exception as err:
-            print(err)
-            raise ValueError("Error importing wordcloud module. Is it installed? If not, install it using: pip install git+git://github.com/amueller/word_cloud.git")
-
         # get the annotations
         if exp is not None:
             # if annotations not yet in experiment - add them
@@ -1499,6 +1528,58 @@ class DBBact(Database):
 
         # calculate the recall, precision, fscore for each term
         fscores, recall, precision, term_count, reduced_f = get_enrichment_score(annotations, sequence_annotations, ignore_exp=ignore_exp, term_info=term_info, threshold=threshold)
+        return fscores, recall, precision, term_count, reduced_f
+
+    def draw_wordcloud(self, exp=None, features=None, term_type='fscore', ignore_exp=None, width=2000, height=1000, freq_weighted=False, relative_scaling=0.5, focus_terms=None, threshold=None, max_id=None):
+        '''Draw a word_cloud for a given set of sequences
+
+        Parameters
+        ----------
+        exp: calour.Experiment or None, optional
+            The experiment containing the sequences (features) of interest.
+            None to not ise the experiment (get the annotations for the features supplied from dbbact and use them)
+        features: list of str or None, optional
+            None to use the features from exp. Otherwise, a list of features ('ACGT' sequences) that is a subset of the features in exp (if exp is supplied)
+            Note: if exp is None, must provide features.
+        term_type: str, optional
+            What score to use for the word cloud. Options are:
+            'recall': sizes are based on the recall (fraction of dbbact term containing annotations that contain the sequences)
+            'precision': sizes are based on the precision (fraction of sequence annotations of the experiment sequences that contain the term)
+            'fscore': a combination of recall and precition (r*p / (r+p))
+        ignore_exp: list of int or None or True, optional
+            List of experiments to ignore in the analysis
+            True to ignore annotations from the current experiment (if exp is supplied)
+            None (default) to use annotations from all experiments including the current one
+        width, height: int, optional
+            The width and heigh of the figure (high values are slower but nicer resolution for publication)
+            If inside a jupyter notebook, use savefig(f, dpi=600)
+        freq_weighted: bool, optional
+            Only when supplying exp
+            True to weight each bacteria by it's mean frequency in exp.data
+            NOT IMPLEMENTED YET!!!
+        relative_scaling: float, optional
+            the effect of the score on the word size. 0.5 is a good compromise (passed to wordcloud.Wordcloud())
+        focus_terms: list of str or None, optional
+            if not None, use only annotations containing all terms in focus_terms list.
+            for example, if focus_terms=['homo sapiens', 'feces'], will only draw the wordcloud for annotations of human feces
+        threshold: float or None, optional
+            if not None, show in word cloud only terms with p-value <= threshold (using the null model of binomial with term freq. as observed in all dbbact)
+        max_id: int or None, optional
+            if not None, limit results to annotation ids <= max_id
+
+
+        Returns
+        -------
+        matplotlib.figure
+        '''
+        import matplotlib.pyplot as plt
+        try:
+            from wordcloud import WordCloud
+        except Exception as err:
+            print(err)
+            raise ValueError("Error importing wordcloud module. Is it installed? If not, install it using: pip install git+git://github.com/amueller/word_cloud.git")
+
+        fscores, recall, precision, term_count, reduced_f = self.get_wordcloud_stats(exp=exp, features=features, ignore_exp=ignore_exp, freq_weighted=freq_weighted, focus_terms=focus_terms, threshold=threshold, max_id=max_id)
 
         logger.debug('draw_cloud for %d words' % len(fscores))
         if len(fscores) == 0:
@@ -1516,9 +1597,8 @@ class DBBact(Database):
 
         # weight by data frequency if needed
         if freq_weighted:
+            raise ValueError('freq weighting not supported yet')
             new_score = defaultdict(float)
-            for cseq in sequence_annotations:
-                pass
 
         # normalize the fractions to a scale max=1
         new_scores = {}
@@ -1549,6 +1629,69 @@ class DBBact(Database):
         fig.tight_layout()
         return fig
 
+    def get_exp_feature_stats(self, exp, ignore_exp=None, term_info=None, term_types='single', threshold=None, max_id=None):
+        '''Get the recall/precision/f-score stats for each feature in the experiment
+
+        Parameters
+        ----------
+        exp: calour.AmpliconExperiment
+        ingore_exp: list of str, optional
+            list of experiment ids to ignore in the analysis
+        term_info: dict of {term (str): details {"total_annotations": float, "total_sequences": float}} (see dbbact rest-api /ontology/get_term_stats) or None, optional
+            The statistics about each term. if None, the function will contact dbbact to get the term_info
+        term_types: list of str
+            the terms to calculate enrichment scores for. options are:
+            'single': each single term (i.e. 'feces')
+            'pairs': all term pairs (i.e. 'feces+homo sapiens')
+        threshold: float or None, optional
+            if not None, return only terms that are significantly enriched in the annotations compared to complete database null with p-val <= threshold
+
+        Returns
+        -------
+        dict of {feature(str): {'fscore':, 'recall':, 'precision': , 'term_count':, 'reduced_f': }}
+        fscore: dict of {term(str): fscore(float)}
+        recall: dict of {term(str): recall(float)}
+        precision: dict of {term(str): precision(float)}
+        term_count: dict of {term(str): total experiments(float)}
+            the number of experiments where annotations for each term appear
+        reduced_f
+        '''
+        # if annotations not yet in experiment - add them
+        self.add_all_annotations_to_exp(exp, max_id=max_id, force=False)
+        # and filter only the ones relevant for features
+        sequence_annotations = exp.databases['dbbact']['sequence_annotations']
+        annotations = exp.databases['dbbact']['annotations']
+        term_info = exp.databases['dbbact']['term_info']
+
+        # get the current experimentID to ignore if ignore_exp is True
+        if ignore_exp is True:
+            ignore_exp = self.db.find_experiment_id(datamd5=exp.info['data_md5'], mapmd5=exp.info['sample_metadata_md5'], getall=True)
+            if ignore_exp is None:
+                logger.warn('No matching experiment found in dbBact. Not ignoring any experiments')
+            else:
+                logger.info('Found %d experiments (%s) matching current experiment - ignoring them.' % (len(ignore_exp), ignore_exp))
+        if ignore_exp is None:
+            ignore_exp = []
+
+        # we need to rekey the annotations with an str (old problem...)
+        annotations = {str(k): v for k, v in annotations.items()}
+
+        res = {}
+        num_seqs_not_found = 0
+        for cseq in exp.feature_metadata.index.values:
+            if cseq not in sequence_annotations:
+                num_seqs_not_found += 1
+                continue
+            annotations_list = sequence_annotations[cseq]
+            fscore, recall, precision, term_count, reduced_f = get_enrichment_score(annotations, [(cseq, annotations_list)], ignore_exp=ignore_exp, term_info=term_info, threshold=threshold)
+            if len(fscore) == 0:
+                continue
+            res[cseq] = {'fscore': fscore, 'precision': precision, 'recall': recall, 'term_count': term_count, 'reduced_f': reduced_f}
+
+        if num_seqs_not_found > 0:
+            logger.warn('%d sequences not found in database data' % num_seqs_not_found)
+        return res
+
     def get_enrichment_score(self, *kargs, **kwargs):
         '''Get f score, recall and precision for set of annotations
 
